// toShare reservation.js
'use strict';

// uses rest.js

class reservations{
	constructor(){
		this.container = document.querySelector('section.reservation');
		this.durationDays = Number(this.container.dataset.durationDays);

		this.buttonPrev = this.container.querySelector('button.prev');
		this.buttonNext = this.container.querySelector('button.next');
		this.inputStartDate = this.container.querySelector('input.start-date');
		this.spanEndDate = this.container.querySelector('span.end-date');
		this.table = this.container.querySelector('table');
		this.defaultColGroup = this.table.querySelector('colgroup').cloneNode(true);
		this.defaultThead = this.table.querySelector('thead').cloneNode(true);
		this.defaultTbody = this.table.querySelector('tbody').cloneNode(true);

		this.onResize = this.onResize.bind(this);
		this.onStartDateChange = this.onStartDateChange.bind(this);
		this.onNextClick = this.onNextClick.bind(this);
		this.onPrevClick = this.onPrevClick.bind(this);
		this.onTableTouchstart = this.onTableTouchstart.bind(this);
		this.onTableTouchmove = this.onTableTouchmove.bind(this);
		this.onTableTouchend = this.onTableTouchend.bind(this);
		this.nextTable = this.nextTable.bind(this);
		this.prevTable = this.prevTable.bind(this);

		window.addEventListener( 'resize', this.onResize);
		this.inputStartDate.addEventListener('change', this.onStartDateChange);
		this.buttonNext.addEventListener('click', this.onNextClick);
		this.buttonPrev.addEventListener('click', this.onPrevClick);
		this.table.addEventListener('touchstart', this.onTableTouchstart);
		this.table.addEventListener('touchmove', this.onTableTouchmove);
		this.table.addEventListener('touchend', this.onTableTouchend);

		this.inputStartDate.value = formatDate((new Date), 'yyyy-MM-dd');
		this.startTouchX = 0;
		this.startTouchScrollX = 0;
		this.currentTouchX = 0;
		this.currentTouchScrollX = 0;
		this.sensitivity = 50;
	}

	onResize(){
		const remPx = parseFloat(getComputedStyle(document.documentElement).fontSize);
		const viewHeight = window.innerHeight;
		const conteinerTop = this.container.getBoundingClientRect().top;
		const containerHeight = viewHeight - conteinerTop - 5 * remPx;
		this.table.style.maxHeight = containerHeight + 'px';
	}
	onStartDateChange(){
		this.table.scrollLeft = 0;
		this.updateLoad();
	}
	onNextClick(){
		const prevLeft = this.table.scrollLeft;
		this.table.scrollLeft += this.table.offsetWidth
		if(Math.abs(this.table.scrollLeft - prevLeft) <= 1){
			this.nextTable();
		}
	}
	onPrevClick(){
		const prevLeft = this.table.scrollLeft;
		this.table.scrollLeft -= this.table.offsetWidth
		if(Math.abs(this.table.scrollLeft - prevLeft) <= 1){
			this.prevTable();
		}
	}
	onTableTouchstart(event){
		this.startTouchX = event.touches[0].clientX;
		this.startTouchScrollX = this.table.scrollLeft;
	}
	onTableTouchmove(event){
		this.currentTouchX = event.touches[0].clientX;
		this.currentTouchScrollX = this.table.scrollLeft;
	}
	onTableTouchend(event){
		const deltaScroll = this.currentTouchScrollX - this.startTouchScrollX;
		const delta = this.currentTouchX - this.startTouchX;
		if(Math.abs(deltaScroll)<=1){
			if(delta < -this.sensitivity){		this.nextTable();}
			else if(delta > this.sensitivity){	this.prevTable();}
		}
	}
	nextTable(){
		var date = new Date(this.inputStartDate.value);
		var drate = this.table.offsetWidth / this.table.scrollWidth *0.8;
		if(drate > 0.5){
			date.setDate(date.getDate() + this.durationDays);
		}else{
			date.setDate(date.getDate() + this.durationDays - this.durationDays * drate);
		}
		this.inputStartDate.value = formatDate((date), 'yyyy-MM-dd');
		this.table.scrollLeft = 0;
		this.updateLoad();
	}
	prevTable(){
		var date = new Date(this.inputStartDate.value);
		var drate = this.table.offsetWidth / this.table.scrollWidth *0.8;
		if(drate > 0.5){
			date.setDate(date.getDate() - this.durationDays);
		}else{
			date.setDate(date.getDate() - this.durationDays + this.durationDays * drate);
		}
		this.inputStartDate.value = formatDate((date), 'yyyy-MM-dd');
		this.table.scrollLeft = 0;
		this.updateLoad();
		this.table.scrollLeft = this.table.scrollWidth - this.table.offsetWidth;
	}


	async updateLoad(){
		const startDate = this.inputStartDate.valueAsDate;
		startDate.setHours(0,0,0);
		const endDate = new Date(startDate);
		endDate.setDate(endDate.getDate() + this.durationDays );
		await atsCalendar.restGet(startDate, endDate);
		await atsReservations.restGet(tableId, startDate, endDate);

		endDate.setDate(endDate.getDate() -1);
		this.spanEndDate.textContent = formatDate(endDate,'～ MM/dd');

		const colGroup = this.defaultColGroup.cloneNode(true);
		const thead = this.defaultThead.cloneNode(true);
		const tbody = this.defaultTbody.cloneNode(true);

		this.updateThead(colGroup, thead);
		this.updateTbody(tbody);

		this.table.innerHTML = '';
		this.table.appendChild(colGroup);
		this.table.appendChild(thead);
		this.table.appendChild(tbody);
		this.updateReservations(tbody);
		this.onResize();
	}

	updateThead(colGroup, thead){
		const theadTr = thead.querySelector('thead tr');
		const theadDate = this.inputStartDate.valueAsDate;
		for(let i = 0; i<= this.durationDays-1; i++){
			const colGroupCol = document.createElement('col');
			const theadTh = document.createElement('th');
			colGroup.appendChild(colGroupCol);
			theadTh.scope="col";
			this.updateTheadTh(theadDate, colGroupCol, theadTh);

			theadTr.appendChild(theadTh);
			theadDate.setDate(theadDate.getDate() + 1);
		}
	}
	updateTheadTh(theadDate, colGroupCol, theadTh){
		const dateType = atsCalendar.dateType(theadDate); // lh sh
		if(dateType !== undefined){
			colGroupCol.classList.add(dateType);
			theadTh.classList.add(dateType);
		}
		theadTh.innerText = ((theadDate.getDate()===1) ? formatDate(theadDate, 'yyyy\nM/') : '') 
			+ formatDate(theadDate, 'd(week)');
	}

	updateTbody(tbody){
		const tbodyTrTemp = document.createElement('tr');
		const tbodyThTemp = document.createElement('th');
		tbodyThTemp.scope='row';
		tbodyTrTemp.appendChild(tbodyThTemp);
		for(let i = 0; i<= this.durationDays-1; i++){
			const tbodyTd = document.createElement('td');
			tbodyTrTemp.appendChild(tbodyTd);
		}

		for(const atsItem of atsItems.data){
			const tbodyTr = tbodyTrTemp.cloneNode(true);
			const tbodyTh = tbodyTr.firstChild;
			this.updateTbodyTh(atsItem, tbodyTh)
			tbody.appendChild(tbodyTr);
		}
	}
	updateTbodyTh(atsItem, tbodyTh){
		tbodyTh.dataset.itemId = atsItem.item_id;
		tbodyTh.innerText = atsItem.item_name;
	}

	updateReservations(tbody){
		const startDate = this.inputStartDate.valueAsDate;
		startDate.setHours(0,0,0);
		const endDate = new Date(startDate);
		endDate.setDate(endDate.getDate() + this.durationDays );

		const aDayWidth = tbody.rows[0].cells[1].getBoundingClientRect().width;
		const aDaySec = 86400;
		const scale = aDayWidth / aDaySec / 1000;

		for(const reservationItem of atsReservations.data){
			for(const tbodyTr of tbody.rows){
				if(Number(tbodyTr.cells[0].dataset.itemId)===reservationItem.item_id){
					// horizontal position & size
					const tbodyP = document.createElement('p');
					const tbodyDiv = document.createElement('div');
					const reservationStartDate = new Date(reservationItem.start_datetime);
					const reservationEndDate = new Date(reservationItem.end_datetime);
					if(reservationStartDate < startDate){reservationStartDate.setTime(startDate.getTime())}
					if(reservationEndDate > endDate){reservationEndDate.setTime(endDate.getTime())}
					const leftDate = new Date(reservationStartDate);
					leftDate.setHours(0,0,0);
					tbodyDiv.style.left = Math.floor((reservationStartDate.getTime() - leftDate.getTime()) * scale) +'px';
					tbodyP.style.width = Math.floor((reservationEndDate.getTime() - reservationStartDate.getTime()) * scale) + 'px';
					// information
					this.updateTbodyData(reservationItem, tbodyP);
					tbodyP.id=reservationItem.reservation_id;
					tbodyP.dataset.reservationId=reservationItem.reservation_id;
					tbodyP.dataset.overlapGroupNo = 0;
					tbodyP.dataset.overlapCount = 0;
					// placed
					const colIndex = Math.floor((reservationStartDate.getTime() - startDate.getTime()) / 1000 / aDaySec) +1;
					tbodyDiv.appendChild(tbodyP);
					tbodyTr.cells[colIndex].appendChild(tbodyDiv);
//					tbodyTr.cells[colIndex].insertBefore(tbodyDiv, tbodyTr.cells[colIndex].firstChild)
				}
			}
		}
		// vertical overlap
		for(const tbodyTr of tbody.rows){
			const tbodyPs = tbodyTr.getElementsByTagName('p');
			const reservationItems = atsReservations.data.filter(d => d.item_id === Number(tbodyTr.cells[0].dataset.itemId));
			let groupCount = 0;
			for(const tbodyP of tbodyPs){
				const aItemIndex = reservationItems.findIndex(i => i.reservation_id === Number(tbodyP.dataset.reservationId));
				const aItem = reservationItems[aItemIndex];
				reservationItems.splice(aItemIndex,1);
				for(const oItem of reservationItems){
					if((aItem.start_datetime <= oItem.start_datetime) & (oItem.start_datetime < aItem.end_datetime)){
						const obodyP = tbodyPs.namedItem(oItem.reservation_id);
						if(Number(tbodyP.dataset.overlapGroupNo)===0){
							groupCount += 1;
							tbodyP.dataset.overlapGroupNo = groupCount;
						}
						obodyP.dataset.overlapGroupNo = tbodyP.dataset.overlapGroupNo;
						obodyP.dataset.overlapCount = Number(tbodyP.dataset.overlapCount) +1;
					}
				}
			}
			// vertical position & size
			for(let groupNo = 1; groupNo <= groupCount; groupNo++){
				const groupPs = Array.from(tbodyPs).filter(p => Number(p.dataset.overlapGroupNo) === groupNo);
				let overlapCount = 0;
				for(const tbodyP of groupPs){
					overlapCount = Math.max(overlapCount, Number(tbodyP.dataset.overlapCount));
				}
				for(const tbodyP of groupPs){
					switch (overlapCount){
						case 1:
							tbodyP.style.top = Number(tbodyP.dataset.overlapCount) *1.6 + 'em';
							tbodyP.style.height = '1.5em';
							break;
						case 2:
							tbodyP.style.top = Number(tbodyP.dataset.overlapCount) *1.1 + 'em';
							tbodyP.style.height = '1em';
							break;
						case 3:
							tbodyP.style.top = Number(tbodyP.dataset.overlapCount) *0.6 + 'em';
							tbodyP.style.height = '0.5em';
							break;
					}
				}
			}
		}
	}


	updateTbodyData(reservationItem, tbodyP){
		const reservationStartDate = new Date(reservationItem.start_datetime);
		const reservationEndDate = new Date(reservationItem.end_datetime);
		tbodyP.innerText = reservationItem.reservation_desc + '\n'
			+ reservationItem.user_name + '\n'
			+ formatDate(reservationStartDate, 'yyyy/MM/dd HH:mm から\n')
			+ formatDate(reservationEndDate, 'yyyy/MM/dd HH:mm まで');
	}

}

const minimumUnitTime = 'time';
const minimumUnitDate = 'date';

class todayView{
	constructor(reservations, minimumUnit){
		// reservation = reservation class or extend reservation class
		this.reservations = reservations;
		this.minimumUnitTime = minimumUnit === minimumUnitTime;
		this.container = document.querySelector('div.today-view');
		this.todayArea = this.container.querySelector('div.today-area');
		this.titleLabel = this.container.querySelector('div.today-title label');
		this.todayList = this.container.querySelector('div.today-list');
		this.buttonCancel = this.container.querySelector('button.cancel');
		this.defaultTable = this.container.querySelector('table').cloneNode(true);
		this.defaultTableTr = this.defaultTable.querySelector('tr').cloneNode(true);

		this.today = new Date();

		this.onClose = this.onClose.bind(this);
		this.onTheadThClick = this.onTheadThClick.bind(this);

		this.buttonCancel.addEventListener('click', this.onClose);

	}
	onClose(){
		this.container.style.display = 'none';
	}
	onTheadThClick(event){
		this.updateList(new Date(event.target.dataset.tooltip));
		this.container.style.display = 'block';
	}
	sameDay(date1, date2) {
		const d1 = new Date(date1);
		const d2 = new Date(date2);

		const year1 = d1.getFullYear();
		const month1 = d1.getMonth();
		const day1 = d1.getDate();

		const year2 = d2.getFullYear();
		const month2 = d2.getMonth();
		const day2 = d2.getDate();

		return year1 === year2 && month1 === month2 && day1 === day2;
	}
	async updateList(today){
		this.today = today;
		this.titleLabel.textContent = formatDate(today, 'yyyy年MM月dd日');

		const startDate = this.today;
		startDate.setHours(0,0,0);
		const endDate = new Date(startDate);
		endDate.setHours(23, 59, 59);
		const todayReservation = new restReservations();
		await todayReservation.restGet(tableId, startDate, endDate);

		this.defaultTable.innerHTML = '';
		this.todayList.innerHTML = '';
		for(const atsItem of atsItems.data){
			const itemTable = this.defaultTable.cloneNode(true);
			const itemCaption = itemTable.createCaption();
			itemCaption.innerText = atsItem.item_name;

			for(const reservationItem of todayReservation.data){
				if(reservationItem.item_id===atsItem.item_id){
					const itemTableTr = this.defaultTableTr.cloneNode(true);
					const td = itemTableTr.querySelectorAll("td");
					const reservationStartDate = new Date(reservationItem.start_datetime);
					const reservationEndDate = new Date(reservationItem.end_datetime);
					let startDateFormat = '';
					let endDateFormat = '';
					if(this.minimumUnitTime){
						startDateFormat = this.sameDay(this.today, reservationStartDate) ? 'HH:mm' : '';
						endDateFormat = this.sameDay(this.today, reservationEndDate) ? ' ～ HH:mm' : ' ～ yyyy/MM/dd';
					}else{
						startDateFormat = this.sameDay(this.today, reservationStartDate) ? '本日' : '';
						endDateFormat = this.sameDay(this.today, reservationEndDate) ? '' : ' ～ yyyy/MM/dd';
					}
					td[0].textContent = formatDate(reservationStartDate, startDateFormat) 
										+ formatDate(reservationEndDate, endDateFormat);
					td[1].innerText = reservationItem.user_name;
					td[2].innerText = reservationItem.reservation_desc;
					itemTable.appendChild(itemTableTr);
				}
			}
			this.todayList.appendChild(itemTable);
		}
	}
}

const NOT_SELECTED = -1;
class reservationEdit{
	constructor(reservations, minimumUnit){
		// reservation = reservation class or extend reservation class
		// minimumUnit = 'date' or 'time'
		this.reservations = reservations;
		this.minimumUnitTime = minimumUnit === minimumUnitTime;
		this.container = document.querySelector('section.edit');

		this.inner = this.container.querySelector('.inner');
		this.reservationId = this.container.querySelector('input.reservation-id');
		this.inputDesc = this.container.querySelector('input.desc');
		this.selectUser = this.container.querySelector('select.user');
		this.selectItem = this.container.querySelector('select.item');
		this.selectTypical = this.container.querySelector('select.typical');
		this.inputStartDate = this.container.querySelector('input.start-date');
		this.inputEndDate = this.container.querySelector('input.end-date');
		if(this.minimumUnitTime){
			this.inputStartTime = this.container.querySelector('input.start-time');
			this.inputEndTime = this.container.querySelector('input.end-time');
		}
		this.buttonAdd = this.container.querySelector('button.add');
		this.buttonFix = this.container.querySelector('button.fix');
		this.buttonDel = this.container.querySelector('button.del');
		this.buttonOpen = this.container.querySelector('.button-open');
		this.buttonClose = this.container.querySelector('.button-close');

		this.onDescChange =  this.onDescChange.bind(this);
		this.onTypicalChange = this.onTypicalChange.bind(this);
		this.onDateTimeChange = this.onDateTimeChange.bind(this);
		this.onExecute = this.onExecute.bind(this);
		this.onInnerOpen = this.onInnerOpen.bind(this);
		this.onInnerClose = this.onInnerClose.bind(this);
		this.onTbodyThClick = this.onTbodyThClick.bind(this);
		this.onTbodyDataClick = this.onTbodyDataClick.bind(this);

		this.inputDesc.addEventListener('input', this.onDescChange);
		this.selectTypical.addEventListener('change', this.onTypicalChange);

		this.inputStartDate.addEventListener('focusout', this.onDateTimeChange);
		this.inputEndDate.addEventListener('focusout', this.onDateTimeChange);
		if(this.minimumUnitTime){
			this.inputStartTime.addEventListener('focusout', this.onDateTimeChange);
			this.inputEndTime.addEventListener('focusout', this.onDateTimeChange);
		}
		this.buttonAdd.addEventListener('click', this.onExecute);
		this.buttonFix.addEventListener('click', this.onExecute);
		this.buttonDel.addEventListener('click', this.onExecute);
		this.buttonOpen.addEventListener('click', this.onInnerOpen);
		this.buttonClose.addEventListener('click', this.onInnerClose);
	}

	updateLoad(){
		this.reservationId.value = 0;

		this.selectUser.innerHTML = '';
		for(const atsUser of atsUsersTable.data){
			let userOption = document.createElement('option');
			userOption.dataset.userId = atsUser.user_id;
			userOption.innerText = atsUser.user_name;
			this.selectUser.appendChild(userOption);
		};

		this.selectUser.selectedIndex = NOT_SELECTED;
		for(const selectOption of this.selectUser){
			if(Number(selectOption.dataset.userId)===atsCurrentUserId){
				this.selectUser.selectedIndex = selectOption.index;
				break;
			}
		}

		this.selectItem.innerHTML = '';
		for(const atsItem of atsItems.data){
			let ItemOption = document.createElement('option');
			ItemOption.dataset.itemId = atsItem.item_id;
			ItemOption.innerText = atsItem.item_name;
			this.selectItem.appendChild(ItemOption);
		};
		this.inputDesc.value = '';
		const now = new Date;
		this.inputStartDate.value = 
		this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
		if(this.minimumUnitTime){
			this.inputStartTime.value = '08:45:00';
			this.inputEndTime.value = '17:00:00';
		}
		this.buttonAdd.disabled = true;
		this.buttonFix.disabled = true;
		this.buttonDel.disabled = true;
	}

	onDescChange(){
		if(Number(this.reservationId.value)===0){
			this.buttonAdd.disabled = (this.inputDesc.value==='');
			this.buttonFix.disabled = true;
			this.buttonDel.disabled = true;
		}else{
			// fix or del
			this.buttonAdd.disabled = true;
			this.buttonFix.disabled = (this.inputDesc.value==='');
			this.buttonDel.disabled = false;
		}
	}

	onTypicalChange(event){
		const now = new Date;
		const selected =  event.target[event.target.selectedIndex];

		if(this.minimumUnitTime){
			switch(selected.dataset.typical){
				case 'today':
					this.inputStartDate.value = 
					this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
					this.inputStartTime.value = '08:45:00';
					this.inputEndTime.value = '17:00:00';
					break;
				case 'am':
					this.inputStartDate.value = 
					this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
					this.inputStartTime.value = '08:45:00';
					this.inputEndTime.value = '12:00:00';
					break;
				case 'pm':
					this.inputStartDate.value = 
					this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
					this.inputStartTime.value = '12:45:00';
					this.inputEndTime.value = '17:00:00';
					break;
			} 
		}else{
			switch(selected.dataset.typical){
				case 'today':
					this.inputStartDate.value = 
					this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
					break;
				case 'week':
					const weekData = new Date;
					weekData.setDate(weekData.getDate() + 7);
					this.inputStartDate.value = formatDate(now, 'yyyy-MM-dd');
					this.inputEndDate.value = formatDate(weekData, 'yyyy-MM-dd');
					break;
				case 'month':
					const monthData = new Date;
					monthData.setMonth(monthData.getMonth() + 1);
					this.inputStartDate.value = formatDate(now, 'yyyy-MM-dd');
					this.inputEndDate.value = formatDate(monthData, 'yyyy-MM-dd');
					break;
			}
		}
		highlight(this.inputStartDate);
		highlight(this.inputEndDate);
		if(this.minimumUnitTime){
			highlight(this.inputStartTime);
			highlight(this.inputEndTime);
		}
	}

	onDateTimeChange(event){
		for(const selectOption of this.selectTypical){
			if(selectOption.dataset.typical === 'antonym'){
				this.selectTypical.selectedIndex = selectOption.index;
			}
		}
		const startDateTime = this.inputStartDate.valueAsDate;
		const endDateTime =	 this.inputEndDate.valueAsDate;
		if(this.minimumUnitTime){
			const startTimeSect = this.inputStartTime.value;
			const startTimeSplit = startTimeSect.split(':');
			const endTimeSect = this.inputEndTime.value;
			const endTimeSplit = endTimeSect.split(':');
			startDateTime.setHours(startTimeSplit[0], startTimeSplit[1]);
			endDateTime.setHours(endTimeSplit[0], endTimeSplit[1]);
		}

		if(startDateTime > endDateTime){
			if(event.target.classList.contains('start-date')){
				endDateTime.setFullYear(startDateTime.getFullYear(),startDateTime.getMonth(),startDateTime.getDate());
				highlight(this.inputEndDate);
			}else{
				startDateTime.setFullYear(endDateTime.getFullYear(),endDateTime.getMonth(),endDateTime.getDate());
				highlight(this.inputStartDate);
			}
		}
		if(this.minimumUnitTime){
			if(startDateTime > endDateTime){
				if(event.target.classList.contains('start-time')){
					endDateTime.setHours(startDateTime.getHours(),startDateTime.getMinutes(),startDateTime.getSeconds());
					highlight(this.inputEndTime);
				}else{
					startDateTime.setHours(endDateTime.getHours(),endDateTime.getMinutes(),endDateTime.getSeconds());
					highlight(this.inputStartTime);
				}
			}
		}
		this.inputStartDate.value = formatDate(startDateTime, 'yyyy-MM-dd');
		this.inputEndDate.value = formatDate(endDateTime, 'yyyy-MM-dd');
		if(this.minimumUnitTime){
			this.inputStartTime.value = formatDate(startDateTime, 'HH:mm:ss');
			this.inputEndTime.value = formatDate(endDateTime, 'HH:mm:ss');
		}
	}

	async onExecute(event){
		const startDateTime = this.inputStartDate.valueAsDate;
		const endDateTime =	 this.inputEndDate.valueAsDate;
		if(!this.minimumUnitTime){
			startDateTime.setHours(0, 0, 1);
			endDateTime.setHours(23, 59, 59);
		}else{
			const startTimeSect = this.inputStartTime.value;
			const startTimeSplit = startTimeSect.split(':');
			const endTimeSect = this.inputEndTime.value;
			const endTimeSplit = endTimeSect.split(':');
			startDateTime.setHours(startTimeSplit[0], startTimeSplit[1]);
			endDateTime.setHours(endTimeSplit[0], endTimeSplit[1]);
		}
		const selectItem = this.selectItem;
		const selectUser = this.selectUser;

		switch(event.target){
			case this.buttonAdd:{
				const doubleBook = await this.checkDoubleBook(startDateTime, endDateTime);
				if(!doubleBook){
					try{
						await atsReservations.restPost(
							Number(selectItem[selectItem.selectedIndex].dataset.itemId),
							Number(selectUser[selectUser.selectedIndex].dataset.userId),
							startDateTime,
							endDateTime,
							this.inputDesc.value
						)
					}catch(e){confirm(e.message);}
				}
				break;
			}
			case this.buttonFix:{
				const doubleBook = await this.checkDoubleBook(startDateTime, endDateTime);
				if(!doubleBook){
					const userId = (selectUser.selectedIndex === NOT_SELECTED)  ? this.selectUser.dataset.userId 
								: Number(selectUser[selectUser.selectedIndex].dataset.userId);
					try{
						await atsReservations.restPut(
							Number(this.reservationId.value),
							Number(selectItem[selectItem.selectedIndex].dataset.itemId),
							userId,
							startDateTime,
							endDateTime,
							this.inputDesc.value
						)
					}catch(e){confirm(e.message);}
				}
				break;
			}
			case this.buttonDel:{
				try{
					await atsReservations.restDelete(
						Number(this.reservationId.value)
					)
				}catch(e){confirm(e.message);}
				break;
			}
			default: confirm('onExecute tagert unknown');
		}
		this.updateLoad();
		this.reservations.updateLoad();
		this.onInnerClose();
	}

	async checkDoubleBook(startDateTime, endDateTime){
		const selectItem = this.selectItem;
		const targetItemId = Number(selectItem[selectItem.selectedIndex].dataset.itemId);
		const targetReservationId = Number(this.reservationId.value);
		const checkReservation = new restReservations();
		await checkReservation.restGet(tableId, startDateTime, endDateTime);
		var doubleBook = false;
		for(const reservationItem of checkReservation.data){
			if(targetItemId === reservationItem.item_id){
				if(targetReservationId !== reservationItem.reservation_id){
					doubleBook = true;
					break;
				}
			}
		}
		if(doubleBook){doubleBook = ! confirm('重複または隣接しています。実行しますか？')}
		return doubleBook;
	}

	onTbodyThClick(event){
		for(const selectOption of this.selectItem){
			if(Number(selectOption.dataset.itemId) === Number(event.target.dataset.itemId)){
				this.onInnerOpen();
				this.selectItem.selectedIndex = selectOption.index;
				highlight(this.selectItem);
			}
		}
	}
	onTbodyDataClick(event){
		if(!event.target.classList.contains('current')){
			for(const item of this.reservations.table.querySelectorAll('tr td p')){
				if(item.classList.contains('current')){item.classList.remove('current');}
			}
			event.target.classList.add('current');
			const reservationItem = atsReservations.findField('reservation_id', Number(event.target.dataset.reservationId));
			const reservationStartDate = new Date(reservationItem.start_datetime);
			const reservationEndDate = new Date(reservationItem.end_datetime);

			this.reservationId.value = reservationItem.reservation_id;
			this.selectUser.dataset.userId = reservationItem.user_id;
			this.selectUser.selectedIndex = NOT_SELECTED;
			for(const selectOption of this.selectUser){
				if(Number(selectOption.dataset.userId)===reservationItem.user_id){
					this.selectUser.selectedIndex = selectOption.index;
				}
			}
			for(const selectOption of this.selectItem){
				if(Number(selectOption.dataset.itemId)===reservationItem.item_id){
					this.selectItem.selectedIndex = selectOption.index;
				}
			}
			this.inputDesc.value = reservationItem.reservation_desc;
			this.inputStartDate.value = formatDate(reservationStartDate, 'yyyy-MM-dd');
			this.inputEndDate.value = formatDate(reservationEndDate, 'yyyy-MM-dd');
			if(this.minimumUnitTime){
				this.inputStartTime.value = formatDate(reservationStartDate, 'HH:mm:ss');
				this.inputEndTime.value = formatDate(reservationEndDate, 'HH:mm:ss');
			}
			this.onInnerOpen();
			this.onDescChange();
			highlight(this.container);
		}else{
			event.target.classList.remove('current');
			this.updateLoad();
			highlight(this.container);
			this.onInnerClose();
		}
	}
	onInnerOpen(){
		this.inner.classList.remove('hidden');
		this.buttonOpen.classList.add('hidden');
		this.buttonClose.classList.remove('hidden');
		this.reservations.onResize();
	}
	onInnerClose(){
		this.inner.classList.add('hidden');
		this.buttonOpen.classList.remove('hidden');
		this.buttonClose.classList.add('hidden');
		this.reservations.onResize();
	}
}
