// toShare items.js
'use strict';
// uses rest, tooltip

const sectionItems = document.querySelector('section.items');
const buttonNew = sectionItems.querySelector('button.new');
const itemTable = sectionItems.querySelector('table');
const itemTbody = sectionItems.querySelector('tbody');
const itemTrTemplate = itemTbody.firstElementChild.cloneNode(true);
const divEditLayer = document.querySelector('div.edit-layer');
const buttonCancel = divEditLayer.querySelector('button.cancel');
const inputItemId = divEditLayer.querySelector('input.item-id');
const inputItemName = divEditLayer.querySelector('input.item-name');
const inputItemDesc = divEditLayer.querySelector('input.item-desc');
const checkboxDisable = divEditLayer.querySelector('input.disable');
const labelDisable = divEditLayer.querySelector('label.disable');
const buttonAdd = divEditLayer.querySelector('button.add');
const buttonFix = divEditLayer.querySelector('button.fix');
const buttonDel = divEditLayer.querySelector('button.del');

async function setTitle(tableId){
	await atsReservationTables.restGet(tableId);
	const atsTable = atsReservationTables.findField('table_id',tableId);
	const pageTitle = `${atsTable.table_name} - ${atsTable.item_name}`;
	document.title = `${pageTitle} - toShare`
	document.querySelector('h1').innerText =  pageTitle;
}

function updateItemList(){
	atsItems.restGet(atsItems.tableId ,WITHDISABLED).then(()=>{
		itemTbody.innerHTML = '';
		for(const atsItem of atsItems.data){
			const itemTr = itemTrTemplate.cloneNode(true);
			itemTr.dataset.itemId = atsItem.item_id;
			if(atsItem.disable){itemTr.classList.add('disable');}
			const td = itemTr.querySelectorAll("td");
			td[0].innerText = atsItem.item_name;
			td[1].innerText = atsItem.item_desc;
			td[2].textContent = (atsItem.disable)? '非表示': '';
			itemTr.addEventListener('click', onItemLineClick);
			itemTbody.appendChild(itemTr);
		}
	})
}
function onResize(){
	const remPx = parseFloat(getComputedStyle(document.documentElement).fontSize);
	const viewHeight = window.innerHeight;
	const conteinerTop = sectionItems.getBoundingClientRect().top;
	const containerHeight = viewHeight - conteinerTop - 6 * remPx;
	itemTable.style.maxHeight = containerHeight + 'px';
}

function onNewEdit(){
		for(const tbodyTr of itemTbody.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		divEditLayer.style.display = 'block';
		inputItemId.value = "0";
		inputItemName.value = "";
		inputItemDesc.value = "";
		checkboxDisable.style.visibility = 'hidden';
		labelDisable.style.visibility = 'hidden';
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = true;
}

function onItemLineClick(event){
	if(!event.currentTarget.classList.contains('current')){
		for(const tbodyTr of itemTbody.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		event.currentTarget.classList.add('current');
		divEditLayer.style.display = 'block';
		const atsItem = atsItems.findField('item_id', Number(event.currentTarget.dataset.itemId));
		inputItemId.value = atsItem.item_id;
		inputItemName.value = atsItem.item_name;
		inputItemDesc.value = atsItem.item_desc;
		checkboxDisable.style.visibility = 'visible';
		labelDisable.style.visibility = 'visible';
		checkboxDisable.checked = (atsItem.disable===1);
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = false;
	}else{
		event.currentTarget.classList.remove('current');
		divEditLayer.style.display = 'none';
	}
}

function onCancel(){
	for(const tbodyTr of itemTbody.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'none';
}

function onItemModified(){
	if(Number(inputItemId.value)===0){
		buttonAdd.disabled = (inputItemName.value==='');
		buttonFix.disabled = true;
		buttonDel.disabled = true;
	}else{
		// fix or del
		buttonAdd.disabled = true;
		buttonFix.disabled = false;
		buttonDel.disabled = false;
	}
}
async function onExecute(event){
	switch(event.target){
		case buttonAdd:{
			try{
				await atsItems.restPost(
					inputItemName.value,
					inputItemDesc.value
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonFix:{
			try{
				await atsItems.restPut(
					Number(inputItemId.value),
					inputItemName.value,
					inputItemDesc.value,
					(checkboxDisable.checked)? 1: 0
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDel:{
			try{
				await atsItems.restDelete(
					Number(inputItemId.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	divEditLayer.style.display = 'none';
	updateItemList();
}


itemTbody.innerHTML = '';

atsItems.tableId = Number(document.location.hash.substring(1));
setTitle(atsItems.tableId);
updateItemList();

window.addEventListener( 'resize', onResize);
buttonNew.addEventListener('click', onNewEdit);
buttonCancel.addEventListener('click', onCancel);
inputItemName.addEventListener('input', onItemModified);
inputItemDesc.addEventListener('input', onItemModified);
checkboxDisable.addEventListener('input', onItemModified);
buttonAdd.addEventListener('click', onExecute);
buttonFix.addEventListener('click', onExecute);
buttonDel.addEventListener('click', onExecute);
onResize();
