// toShare reservations.js
'use strict';
// uses rest, tooltip

const sectionReservations = document.querySelector('.reservations');
const buttonNew = sectionReservations.querySelector('button.new');
const tableReservation = sectionReservations.querySelector('table');
const tbodyReservation = sectionReservations.querySelector('tbody');
const trTemplateReservation = tbodyReservation.firstElementChild.cloneNode(true);
const divEditLayer = document.querySelector('div.edit-layer');
const buttonCancel = divEditLayer.querySelector('button.cancel');
const inputTableId = divEditLayer.querySelector('input.table-id');
const inputTableName = divEditLayer.querySelector('input.table-name');
const selectTableUnit = divEditLayer.querySelector('select.table-unit');
const inputTableUri = divEditLayer.querySelector('input.table-custom-uri');
const inputTableDesc = divEditLayer.querySelector('input.table-desc');
const inputItemName = divEditLayer.querySelector('input.item-name');
const checkboxDisable = divEditLayer.querySelector('input.disable');
const labelDisable = divEditLayer.querySelector('label.disable');
const buttonAdd = divEditLayer.querySelector('button.add');
const buttonFix = divEditLayer.querySelector('button.fix');
const buttonDel = divEditLayer.querySelector('button.del');

function updateReservationList(){
	atsReservationTables.restGet(TABLE_ID_ALL, WITHDISABLED).then(()=>{
		tbodyReservation.innerHTML = '';
		for(const atsTable of atsReservationTables.data){
			const aTag = document.createElement('a');
			aTag.href = 'items';
			aTag.innerText = '(編集)'  + atsTable.item_name;
			aTag.hash = atsTable.table_id;
			const trTable = trTemplateReservation.cloneNode(true);
			trTable.dataset.tableId = atsTable.table_id;
			if(atsTable.disable){trTable.classList.add('disable');}
			const td = trTable.querySelectorAll('td');
			td[0].innerText = atsTable.table_name;
			td[1].innerText = (atsTable.table_custom === 0) ? selectTableUnit.namedItem(atsTable.table_uri).label : atsTable.table_uri;
			td[2].innerText = atsTable.table_desc;
			td[3].textContent = '';
			td[3].appendChild(aTag);
			td[4].textContent = (atsTable.disable)? '非表示' : '';
			trTable.addEventListener('click', onTrReservationClick);
			tbodyReservation.appendChild(trTable);
		}
	})
}
function onResize(){
	const remPx = parseFloat(getComputedStyle(document.documentElement).fontSize);
	const viewHeight = window.innerHeight;
	const conteinerTop = sectionReservations.getBoundingClientRect().top;
	const containerHeight = viewHeight - conteinerTop - 6 * remPx;
	tableReservation.style.maxHeight = containerHeight + 'px';
}
function onNewEdit(){
		for(const tbodyTr of tbodyReservation.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		divEditLayer.style.display = 'block';
		inputTableId.value = '0';
		inputTableName.value = '';
		selectTableUnit.slectedIndex = 1;
		inputTableUri.value = '';
		inputTableUri.style.visibility = 'hidden';
		inputTableDesc.value = '';
		inputItemName.value = 'アイテム';
		checkboxDisable.style.visibility = 'hidden';
		labelDisable.style.visibility = 'hidden';
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = true;
}
function onTrReservationClick(event){
	if((event.target.tagName != 'A')&(!event.currentTarget.classList.contains('current'))){
		for(const tbodyTr of tbodyReservation.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		event.currentTarget.classList.add('current');
		divEditLayer.style.display = 'block';
		const atsTable = atsReservationTables.findField('table_id', Number(event.currentTarget.dataset.tableId));
		inputTableId.value = atsTable.table_id;
		inputTableName.value = atsTable.table_name;
		selectTableUnit.selectedIndex = 1;
		if(atsTable.table_custom===0){
			selectTableUnit.selectedIndex = selectTableUnit.namedItem(atsTable.table_uri).index;
			inputTableUri.value = '';
			inputTableUri.style.visibility = 'hidden';
		}else{
			selectTableUnit.selectedIndex = selectTableUnit.namedItem('custom').index;
			inputTableUri.value = atsTable.table_uri;
			inputTableUri.style.visibility = 'visible';
		}
		inputTableDesc.value = atsTable.table_desc;
		inputItemName.value = atsTable.item_name;
		checkboxDisable.style.visibility = 'visible';
		labelDisable.style.visibility = 'visible';
		checkboxDisable.checked = (atsTable.disable===1);
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = false;
	}else{
		event.currentTarget.classList.remove('current');
		divEditLayer.style.display = 'none';
	}
}
function onCancel(){
	for(const tbodyTr of tbodyReservation.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'none';
}
function onTableModified(){
	if(Number(inputTableId.value)===0){
		buttonAdd.disabled = (inputTableName.value==='');
		buttonFix.disabled = true;
		buttonDel.disabled = true;
	}else{
		// fix or del
		buttonAdd.disabled = true;
		buttonFix.disabled = false;
		buttonDel.disabled = false;
	}
}
function onTableUnitChange(event){
		const selected =  event.target[event.target.selectedIndex];
		inputTableUri.style.visibility = (selected.id != 'custom') ? 'hidden' : 'visible';
		onTableModified();
}
async function onExecute(event){
	let tableCustom = 0;
	let tableUri = '';
	if(selectTableUnit[selectTableUnit.selectedIndex].id !='custom'){
		tableCustom = 0;
		tableUri = selectTableUnit[selectTableUnit.selectedIndex].id;
	}else{
		tableCustom = 1;
		tableUri = inputTableUri.value;
	}
	switch(event.target){
		case buttonAdd:{
			try{
				await atsReservationTables.restPost(
					inputTableName.value,
					tableCustom, tableUri,
					inputTableDesc.value,
					inputItemName.value
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonFix:{
			try{
				await atsReservationTables.restPut(
					Number(inputTableId.value),
					inputTableName.value,
					tableCustom, tableUri,
					inputTableDesc.value,
					inputItemName.value,
					(checkboxDisable.checked)? 1: 0
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDel:{
			try{
				await atsReservationTables.restDelete(
					Number(inputTableId.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	divEditLayer.style.display = 'none';
	updateReservationList();
}

tbodyReservation.innerHTML = '';
updateReservationList();
window.addEventListener( 'resize', onResize);
buttonNew.addEventListener('click', onNewEdit);
buttonCancel.addEventListener('click', onCancel);
inputTableName.addEventListener('input', onTableModified);
selectTableUnit.addEventListener('change', onTableUnitChange);
inputTableDesc.addEventListener('input', onTableModified);
inputItemName.addEventListener('input', onTableModified);
checkboxDisable.addEventListener('change', onTableModified);
buttonAdd.addEventListener('click', onExecute);
buttonFix.addEventListener('click', onExecute);
buttonDel.addEventListener('click', onExecute);
onResize();
