// toShare users.js
'use strict';
// uses rest, tooltip

const sectionUsers = document.querySelector('section.users');
const buttonNew = sectionUsers.querySelector('button.new');
const userTable = sectionUsers.querySelector('table');
const userTbody = sectionUsers.querySelector('tbody');
const userTrTemplate = userTbody.firstElementChild.cloneNode(true);
const divEditLayer = document.querySelector('div.edit-layer');
const buttonCancel = divEditLayer.querySelector('button.cancel');
const inputUserId = divEditLayer.querySelector('input.user-id');
const inputUserAccount = divEditLayer.querySelector('input.user-account');
const inputUserName = divEditLayer.querySelector('input.user-name');
const inputUserDesc = divEditLayer.querySelector('input.user-desc');
const inputUserSeries = divEditLayer.querySelector('input.user-series');
const checkboxDisable = divEditLayer.querySelector('input.disable');
const labelDisable = divEditLayer.querySelector('label.disable');
const buttonAdd = divEditLayer.querySelector('button.add');
const buttonFix = divEditLayer.querySelector('button.fix');
const buttonDel = divEditLayer.querySelector('button.del');

function updateUserList(){
	atsUsersTable.restGet(USER_ID_ALL, WITHDISABLED).then(()=>{
		userTbody.innerHTML = '';
		for(const atsUser of atsUsersTable.data){
			const userTr = userTrTemplate.cloneNode(true);
			const td = userTr.querySelectorAll("td");
			userTr.dataset.userId = atsUser.user_id;
			if(atsUser.disable){userTr.classList.add('disable');}
			td[0].innerText = atsUser.user_account;
			td[1].innerText = atsUser.user_name;
			td[2].innerText = atsUser.user_desc;
			td[3].innerText = atsUser.user_series;
			td[4].textContent = (atsUser.disable)? '非表示' : '';
			userTr.addEventListener('click', onUserLineClick);
			userTbody.appendChild(userTr);
		}
	})
}
function onResize(){
	const remPx = parseFloat(getComputedStyle(document.documentElement).fontSize);
	const viewHeight = window.innerHeight;
	const conteinerTop = sectionUsers.getBoundingClientRect().top;
	const containerHeight = viewHeight - conteinerTop - 6 * remPx;
	userTable.style.maxHeight = containerHeight + 'px';
}
function onNewEdit(){
		for(const tbodyTr of userTbody.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		divEditLayer.style.display = 'block';
		inputUserId.value = "0";
		inputUserAccount.value = "";
		inputUserName.value = "";
		inputUserDesc.value = "";
		inputUserSeries.value = "";
		checkboxDisable.style.visibility = 'hidden';
		labelDisable.style.visibility = 'hidden';
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = true;
}

function onUserLineClick(event){
	if(!event.currentTarget.classList.contains('current')){
		for(const tbodyTr of userTbody.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		event.currentTarget.classList.add('current');
		divEditLayer.style.display = 'block';
		const atsUser = atsUsersTable.findField('user_id', Number(event.currentTarget.dataset.userId));
		inputUserId.value = atsUser.user_id;
		inputUserAccount.value = atsUser.user_account;
		inputUserName.value = atsUser.user_name;
		inputUserDesc.value = atsUser.user_desc;
		inputUserSeries.value = atsUser.user_series;
		checkboxDisable.style.visibility = 'visible';
		labelDisable.style.visibility = 'visible';
		checkboxDisable.checked = (atsUser.disable===1);
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = false;
	}else{
		event.currentTarget.classList.remove('current');
		divEditLayer.style.display = 'none';
	}
}

function onCancel(){
	for(const tbodyTr of userTbody.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'none';
}
function onUserModified(){
	if(Number(inputUserId.value)===0){
		buttonAdd.disabled = (inputUserAccount.value==='') || (inputUserName.value==='');
		buttonFix.disabled = true;
		buttonDel.disabled = true;
	}else{
		// fix or del
		buttonAdd.disabled = true;
		buttonFix.disabled = false;
		buttonDel.disabled = false;
	}
}
async function onExecute(event){
	switch(event.target){
		case buttonAdd:{
			try{
				await atsUsersTable.restPost(
					inputUserAccount.value,
					inputUserName.value,
					inputUserDesc.value,
					inputUserSeries.value
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonFix:{
			try{
				await atsUsersTable.restPut(
					Number(inputUserId.value),
					inputUserAccount.value,
					inputUserName.value,
					inputUserDesc.value,
					inputUserSeries.value,
					(checkboxDisable.checked)? 1: 0
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDel:{
			try{
				await atsUsersTable.restDelete(
					Number(inputUserId.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	divEditLayer.style.display = 'none';
	updateUserList();
}

userTbody.innerHTML = '';
updateUserList();
window.addEventListener( 'resize', onResize);
buttonNew.addEventListener('click', onNewEdit);
buttonCancel.addEventListener('click', onCancel);
inputUserAccount.addEventListener('input', onUserModified);
inputUserName.addEventListener('input', onUserModified);
inputUserDesc.addEventListener('input', onUserModified);
inputUserSeries.addEventListener('input', onUserModified);
checkboxDisable.addEventListener('change', onUserModified);
buttonAdd.addEventListener('click', onExecute);
buttonFix.addEventListener('click', onExecute);
buttonDel.addEventListener('click', onExecute);
onResize();
